#include "deepinmultimedia.h"
#include "osinfo.h"

#include <QDBusPendingCall>
#include <QDBusReply>
#include <QLoggingCategory>
#include <QRegularExpression>
#include <QProcess>
#include <QThread>

Q_DECLARE_LOGGING_CATEGORY(logOsControl)

using namespace uos_ai;

// DBus 服务名称和路径定义
#ifdef COMPILE_ON_V23
static const QString MUSIC_SERVICE = "org.deepin.dde.Music1";
static const QString MUSIC_PATH = "/org/deepin/dde/Music1";
static const QString MUSIC_INTERFACE = "org.deepin.dde.Music1";
#else
static const QString MUSIC_SERVICE = "org.mpris.MediaPlayer2.DeepinMusic";
static const QString MUSIC_PATH = "/org/mpris/speech";
static const QString MUSIC_INTERFACE = "com.deepin.speech";
#endif

DeepinMultimedia::DeepinMultimedia(QObject *parent) : QObject(parent)
{
    // 初始化音乐播放器 DBus 接口
    m_musicProxy.reset(new QDBusInterface(
        MUSIC_SERVICE,
        MUSIC_PATH,
        MUSIC_INTERFACE,
        QDBusConnection::sessionBus(), this));
}

DeepinMultimedia::~DeepinMultimedia()
{
}

bool DeepinMultimedia::callMusicDBusMethod(QDBusInterface *interface,
                                    const QString &method,
                                    const QVariantList &args,
                                    QString &errorInfo)
{
    // 如果接口无效，尝试启动音乐应用
    if (!interface || !interface->isValid()) {
        // 启动音乐应用进程
        QProcess process;
        process.setProcessEnvironment(UosInfo()->pureEnvironment());
        process.setProgram("/usr/bin/deepin-music");
        qint64 pid = 0;
        bool execResult = process.startDetached(&pid);
        
        if (!execResult) {
            errorInfo = process.errorString();
            return false;
        }
        
        // 等待进程启动和DBus服务就绪
        int retryCount = 0;
        const int maxRetries = 3;
        while (retryCount < maxRetries) {
            QThread::msleep(500); // 等待500毫秒
            
            // 尝试创建DBus接口
            m_musicProxy.reset(new QDBusInterface(
                MUSIC_SERVICE,
                MUSIC_PATH,
                MUSIC_INTERFACE,
                QDBusConnection::sessionBus(), this));
                
            if (m_musicProxy && m_musicProxy->isValid()) {
                interface = m_musicProxy.data();
                break;
            }
            
            retryCount++;
        }
        
        if (retryCount >= maxRetries) {
            errorInfo = "Failed to establish DBus connection after multiple attempts";
            return false;
        }
    }

    if (!interface || !interface->isValid()) {
        errorInfo = "DBus interface is not valid";
        return false;
    }

    QDBusReply<QVariant> reply = interface->callWithArgumentList(QDBus::Block, "invoke", QVariantList() << method << args.value(0, ""));
    if (!reply.isValid()) {
        errorInfo = reply.error().message();
        return false;
    }

    return true;
}

bool DeepinMultimedia::playOneSong(const QString &name, const QString &singer,
                                 const QString &album, const QString &songList,
                                 int index, QString &errorInfo)
{
    // 构建参数列表
    QStringList keys;
    QMap<QString, QString> arguments;
    
    if (!name.isEmpty()) {
        keys << "name";
        arguments["name"] = name;
    }
    if (!singer.isEmpty()) {
        keys << "singer";
        arguments["singer"] = singer;
    }
    if (!album.isEmpty()) {
        keys << "album";
        arguments["album"] = album;
    }
    if (!songList.isEmpty()) {
        keys << "songList";
        arguments["songList"] = songList;
    }
    if (index >= 0) {
        keys << "index";
        arguments["index"] = QString::number(index);
    }

    // 根据参数数量选择不同的调用方式
    switch (keys.size()) {
    case 0:
        return callMusicDBusMethod(m_musicProxy.data(), "playMusic", QVariantList() << "", errorInfo);
    case 1: {
        QString key = keys[0];
        if (key == "name") {
            return callMusicDBusMethod(m_musicProxy.data(), "playMusic", QVariantList() << arguments[key], errorInfo);
        } else if (key == "album") {
            return callMusicDBusMethod(m_musicProxy.data(), "playAlbum", QVariantList() << arguments[key], errorInfo);
        } else if (key == "singer") {
            return callMusicDBusMethod(m_musicProxy.data(), "playArtist", QVariantList() << arguments[key], errorInfo);
        } else if (key == "songList") {
            return callMusicDBusMethod(m_musicProxy.data(), "playSonglist", QVariantList() << arguments[key], errorInfo);
        } else if (key == "index") {
            QRegularExpression rx("(\\d+)");
            QRegularExpressionMatchIterator i = rx.globalMatch(arguments[key]);
            QString indexStr;
            if (i.hasNext()) {
                QRegularExpressionMatch match = i.next();
                indexStr = match.captured(1);
            }
            if (indexStr.toInt() == 0) {
                indexStr = "1";
            }
            if (!indexStr.isEmpty() && indexStr.toInt() > 0) {
                return callMusicDBusMethod(m_musicProxy.data(), "playIndex", QVariantList() << indexStr, errorInfo);
            }
        }
        break;
    }
    case 2: {
        if (keys.contains("singer") && keys.contains("name")) {
            QString param = arguments["singer"] + ":" + arguments["name"];
            return callMusicDBusMethod(m_musicProxy.data(), "playArtistMusic", QVariantList() << param, errorInfo);
        }
        break;
    }
    default:
        break;
    }

    errorInfo = "Invalid parameter combination";
    return false;
}

bool DeepinMultimedia::stateControl(const QString &control, QString &errorInfo)
{
    if (control == "Play") {
        return callMusicDBusMethod(m_musicProxy.data(), "resume", QVariantList(), errorInfo);
    } else if (control == "Pause") {
        return callMusicDBusMethod(m_musicProxy.data(), "pause", QVariantList(), errorInfo);
    } else if (control == "Stop") {
        return callMusicDBusMethod(m_musicProxy.data(), "stop", QVariantList(), errorInfo);
    } else if (control == "Previous") {
        return callMusicDBusMethod(m_musicProxy.data(), "pre", QVariantList(), errorInfo);
    } else if (control == "Next") {
        return callMusicDBusMethod(m_musicProxy.data(), "next", QVariantList(), errorInfo);
    } else if (control.contains("ListLoop")) {
        return callMusicDBusMethod(m_musicProxy.data(), "setMode", QVariantList() << "0", errorInfo);
    } else if (control.contains("SingleLoop")) {
        return callMusicDBusMethod(m_musicProxy.data(), "setMode", QVariantList() << "1", errorInfo);
    } else if (control.contains("Shuffle")) {
        return callMusicDBusMethod(m_musicProxy.data(), "setMode", QVariantList() << "2", errorInfo);
    }

    errorInfo = "Invalid control command";
    return false;
}

bool DeepinMultimedia::seek(const QString &position, QString &errorInfo)
{
    QRegularExpression rx("(\\d+)");
    QRegularExpressionMatchIterator i = rx.globalMatch(position);
    QStringList times;

    while (i.hasNext()) {
        QRegularExpressionMatch match = i.next();
        times.append(match.captured(1));
    }

    if (times.isEmpty()) {
        errorInfo = "Invalid position format";
        return false;
    }

    QString msec;
    if (times.size() == 1) {
        msec = QString::number(times[0].toInt() * 1000);
    } else if (times.size() == 2) {
        msec = QString::number((times[0].toInt() * 60 + times[1].toInt()) * 1000);
    } else {
        errorInfo = "Invalid time format";
        return false;
    }

    return callMusicDBusMethod(m_musicProxy.data(), "setPosition", QVariantList() << msec, errorInfo);
}

bool DeepinMultimedia::playMyFavorite(QString &errorInfo)
{
    return callMusicDBusMethod(m_musicProxy.data(), "playFaverite", QVariantList(), errorInfo);
}

bool DeepinMultimedia::addRemoveFavorite(const QString &addFavorite,
                                       const QString &removeFavorite,
                                       QString &errorInfo)
{
    if (!addFavorite.isEmpty()) {
        return callMusicDBusMethod(m_musicProxy.data(), "addFaverite", QVariantList(), errorInfo);
    } else if (!removeFavorite.isEmpty()) {
        return callMusicDBusMethod(m_musicProxy.data(), "removeFaverite", QVariantList(), errorInfo);
    }

    errorInfo = "No action specified";
    return false;
}
